@@ -23,6 +23,10 @@
 #include <stddef.h>
 #include <stdint.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 // -----------------------------------------------------------------------------
 // NOTE:
 // Ensure that symbols in zdnn.h and zdnn.map are in sync!
@@ -370,11 +374,11 @@ typedef struct zdnn_ztensor {
   char reserved2[20]; // not currently used, should contain zeros.
 } zdnn_ztensor;
 
-#define ZDNN_VERSION "1.1.1"
-#define ZDNN_VERNUM 0x010101 // 0x[major][minor][patch]
+#define ZDNN_VERSION "1.1.2"
+#define ZDNN_VERNUM 0x010102 // 0x[major][minor][patch]
 #define ZDNN_VER_MAJOR 1
 #define ZDNN_VER_MINOR 1
-#define ZDNN_VER_PATCH 1
+#define ZDNN_VER_PATCH 2
 
 // -----------------------------------------------------------------------------
 // External Tensor Functions
@@ -649,5 +653,8 @@ zdnn_status zdnn_get_max_limit(zdnn_data_types transformed_type,
 zdnn_status zdnn_get_min_limit(zdnn_data_types transformed_type,
                                zdnn_data_types pre_transformed_type,
                                void *limit);
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
 
 #endif /* ZDNN_ZDNN_H_ */
